/* ----------------------------------------------------------------------------
   Программа, иллюстрирующая использование динамической загрузки
   разделяемой библиотеки.
---------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

/* Включать этот заголовочный файл теперь уже не нужно, т. к. при компиляции
   программы функции из библиотеки еще недоступны.
   #include "geometry.h" */

/* Определим указатель на функцию, принимающую один параметр
   и возвращающую значение типа float
   (таковы все функции из нашей библиотеки). */
typedef float ( *library_func )( float );

/* прототип функции get_func() */
library_func get_func( void *lib_handle, const char *func_name );

int main( void )
{
  float rad;
  float side;

  void *lib;             /* дескриптор загружаемой библиотеки */
  library_func lib_fn;   /* указатель на функцию из библиотеки */
  const char *error;     /* сообщение об ошибке при обращении к библиотеке */
  
  /* Открываем разделяемую библиотеку.
     ПРИМЕЧАНИЕ. Можно в качестве имени библиотеки указать не только
                 libgeometry.so, но и libgeometry.so.2, и
                 libgeometry.so.2.0.1, и libgeometry.so.2.1.1.
                 Можно указать даже libgeometry.so.1, и это будет работать.
                 Рекомендуем поэкспериментировать с именами библиотеки.
                 Такая гибкость возможна в том числе потому, что часть
                 упоминаемых имен библиотеки являются символическими
                 ссылками. */
  lib = dlopen( "libgeometry.so", RTLD_LAZY );

  if ( !lib )
  {
    fprintf( stderr, "Не могу открыть библиотеку libgeometry.so: %s\n",
             dlerror() );
    exit( 1 );
  }                 

  /* DEBUG */
  /* dlsym( lib, "nonexistent_function" );
     dlsym( lib, "circle_len" );
     if ( error = dlerror() )
       fprintf( stderr, "Ошибка: %s\n", error );
    
     fprintf( stderr, "Ошибка: %s\n", dlerror() );
  */
  
  printf( "Динамическая загрузка разделяемой библиотеки\n\n" );

  printf( "Введите радиус окружности: " );
  scanf( "%f", &rad );
  printf( "\n%f\n", rad );

  /* Если удалось получить указатель на функцию circle_len,
     то выполним расчет.
     ПРИМЕЧАНИЕ. Вызываем библиотечную функцию, используя указатель на нее. */
  if ( ( lib_fn = get_func( lib, "circle_len" ) ) != NULL )
    printf( "Длина окружности: %f\n", ( *lib_fn )( rad ) );
  
  /* Получаем доступ к остальным функциям из библиотеки аналогично. */
  if ( ( lib_fn = get_func( lib, "circle_area" ) ) != NULL )
    printf( "Площадь круга: %f\n", ( *lib_fn )( rad ) );

  printf( "\nВведите длину стороны квадрата: " );
  scanf( "%f", &side );
  printf( "\n%f\n", side );

  if ( ( lib_fn = get_func( lib, "square_perim" ) ) != NULL )
    printf( "Периметр квадрата: %f\n", ( *lib_fn )( side ) );

  if ( ( lib_fn = get_func( lib, "square_area" ) ) != NULL )
    printf( "Площадь квадрата: %f\n", ( *lib_fn )( side ) );
  
  /* Закрываем библиотеку. */
  dlclose( lib );

  return 0;
}

library_func get_func( void *lib_handle, const char *func_name )
{
  /* lib_handle - дескриптор загружаемой библиотеки */
  /* func_name - имя функции из библиотеки */

  library_func lib_func; /* указатель на функцию из библиотеки */
  const char *error;

  /* Вызываем эту функцию, чтобы на всякий случай гарантированно "сбросить"
     флаг наличия ошибки. */
  /* ПРИМЕЧАНИЕ. См. закомментированный фрагмент с пометкой DEBUG
                 в функции main(). Можно раскомментировать его,
                 откомпилировать программу и посмотреть, что получится. */
  dlerror();

  /* Получим адрес требуемой библиотечной функции. */
  lib_func = dlsym( lib_handle, func_name );

  /* Обратите внимание, что здесь используется оператор присваивания,
     а не проверки на равенство. */
  if ( error = dlerror() )
  {
    fprintf( stderr, "Не могу найти %s: %s\n", func_name, error );

    /* Можно при необходимости и вообще завершить программу. */
    /* exit( 1 ); */

    return NULL;
  }

  return lib_func;
}
