/*
 * Программа: test.c
 * Проверка возможности доступа к базе данных с помощью библиотеки libpq
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo; /* параметры для соединения с базой данных */
  PGconn     *conn;     /* дескриптор соединения с базой данных */
  PGresult   *res;      /* результат выполнения SQL-запроса */
  int        nFields;   /* число полей в выборке */
  int        i, j;

  /*
   * если пользователь передает параметр в командной строке,
   * то нужно использовать его в качестве строки conninfo;
   * если параметр в командной строке не передан, то по умолчанию
   * для подключения к базе данных используется строка
   * "dbname=test user=postgres"
  */
  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /* подключаемся к базе данных */
  conn = PQconnectdb( conninfo );

  /* проверяем успешность подключения */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "Подключиться к базе данных не удалось: %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /* функция PQexec исполняет SQL-запрос, переданный ей
     в виде символьной строки */
  res = PQexec( conn, "SELECT * FROM Students" );
  if (PQresultStatus(res) != PGRES_TUPLES_OK)
  {
    fprintf( stderr, "Запрос SELECT неудачен: %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /* количество полей в выборке */
  
  /* сначала выведем имена полей для полученной выборки 
     (фактически это имена полей таблицы Students) */
  for ( j = 0; j < nFields; j++ )
  {
    /* для поля "Фамилия, имя, отчество" выделим больше места
       (функция PQfname возвращает имя поля по его порядковому
       номеру в выборке) */
    if ( strcmp( PQfname( res, j ), "name" ) == 0 )
      printf( "%-30s", PQfname( res, j ) );
    else
      printf( "%-10s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /* выведем данные из полученной выборки
     (функция PQntuples возвращает число строк в выборке) */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "name" ) == 0 )
        /* функция PQgetvalue возвращает значение поля номер i
           из строки номер j */
        printf( "%-30s", PQgetvalue( res, i, j ) );
      else
        printf( "%-10s", PQgetvalue( res, i, j ) );
    }
    printf( "\n" );
  }

  /* освободим память, которую занимала структура res,
     как только эта структура больше не нужна */
  PQclear( res );

  /* закроем соединение с базой данных */
  PQfinish( conn );

  return 0;
}
