/*
 * Программа: test2.c
 * Проверка возможности доступа к базе данных с помощью библиотеки libpq.
 * Выборка производится из двух таблиц: главной и подчиненной, когда для
 * каждой записи из главной таблицы выбираются все записи, связанный с ней
 * по внешнему ключу, из подчиненной таблицы. Главная таблица -- students,
 * подчиненная -- progress. Команды для создания таблиц см. в файле
 * test_db.sql. Для создания таблиц можно поступить так: 
 *
 * psql -f test_db.sql -U postgres
 */

/*
  ВНИМАНИЕ. При использовании в базе данных символов русского алфавита нужно
  учитывать следующее. Функция printf() рассчитана на однобайтовые символы,
  но в кодировке UTF-8, используемой в ОС Debian, русские символы состоят
  из двух байтов. Конечно, при выводе русского символа на экран он занимает
  одну позицию, но внутри -- два байта. Как это проявляется при выводе
  данных? А вот как. Например, мы в форматной строке функции printf() 
  указываем "%-30s", чтобы значение было выровнено по левому краю (на это 
  указывает символ "-") и занимало колонку шириной 30 символов. При выводе
  значения "Физика" из таблицы progress получается следующее: в слове "Физика"
  6 букв русского алфавита, т е. 12 байтов. Эти 12 байтов дополняются пробелами
  до суммарной ширины 30 байтов, т. е. добавляется 18 пробелов. Визуально
  ширина этого поля получится равной 6 + 18 = 24 байтам, а "внутри" - 
  12 + 18 = 30 байтов. Для значения "Математика" получим: 10 * 2 = 20 байтов
  внутреннее представление, для дополнения до 30 байтов функция printf()
  добавит 10 пробелов, поэтому на экране это поле будет иметь ширину 
  20 позиций, что меньше, чем для более КОРОТКОГО значения "Физика". Парадокс?
  Но он объясним. При выводе данных из таблицы students имеет место такая же
  картина. Поэтому данные в колонках "Номер паспорта" и "Серия паспорта"
  оказываются невыровненными.
    Для того, чтобы все было и правильно, и красиво, нужно использовать 
  либо широкие символы, либо многобайтовые кодировки. Примеры можно посмотреть
  в программе prg18.c в кратком руководстве по языку программирования C
  (http://www.morgunov.org/teaching.html). Поскольку для реализации корректной
  работы с символами Unicode требуется приложить некоторые усилия, то можно,
  в принципе, на начальном этапе изучения PostgreSQL и libpq обойтись и так:
  либо в базе данных использовать только латинский алфавит, либо при 
  использовании русского алфавита мириться с неизбежными искажениями ширины
  выводимых столбцов данных.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo; /* параметры для соединения с базой данных */
  PGconn     *conn;     /* дескриптор соединения с базой данных */
  PGresult   *res_s;    /* результат выполнения SQL-запроса к табл. students */
  PGresult   *res_p;    /* результат выполнения SQL-запроса к табл. progress */ 
  int        nFields_s; /* число полей в выборке из табл. students */
  int        nFields_p; /* число полей в выборке из табл. progress */

  /* индексы для циклов обработки выборок из таблиц */
  int        i, j;      /* students */ 
  int        k, l;      /* progress */

  /* команда для выборки из табл. progress */
  char command_p[ 256 ];

  /*
   * Если пользователь передает параметр в командной строке,
   * то нужно использовать его в качестве строки conninfo;
   * если параметр в командной строке не передан, то по умолчанию
   * для подключения к базе данных используется строка
   * "dbname=test user=postgres"
  */
  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /* Подключаемся к базе данных. */
  conn = PQconnectdb( conninfo );

  /* Проверяем успешность подключения. */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "Подключиться к базе данных не удалось: %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /* Функция PQexec исполняет SQL-запрос, переданный ей
     в виде символьной строки. */
  res_s = PQexec( conn, "SELECT * FROM students" );

  if ( PQresultStatus( res_s ) != PGRES_TUPLES_OK)
  {
    fprintf( stderr, "Запрос SELECT к таблице students неудачен: %s",
             PQerrorMessage( conn ) );
    PQclear( res_s );
    exit_nicely( conn );
  }

  nFields_s = PQnfields( res_s );  /* количество полей в выборке */
  
  /* Сначала выведем имена полей для полученной выборки 
     (фактически это имена полей таблицы students). */
  for ( j = 0; j < nFields_s; j++ )
  {
    /* Для поля "Фамилия, имя, отчество" выделим больше места
       (функция PQfname возвращает имя поля по его порядковому
       номеру в выборке). */
    if ( strcmp( PQfname( res_s, j ), "name" ) == 0 )
      printf( "%-40s", PQfname( res_s, j ) );
    else
      printf( "%-12s", PQfname( res_s, j ) );
  }

  printf( "\n\n" );

  /* Выведем данные из полученной выборки
     (функция PQntuples возвращает число строк в выборке). */
  for ( i = 0; i < PQntuples( res_s ); i++ )
  {
    for ( j = 0; j < nFields_s; j++ )
    {
      if ( strcmp( PQfname( res_s, j ), "name" ) == 0 )
        /* Функция PQgetvalue возвращает значение поля номер i
           из строки номер j. */
        printf( "%-40s", PQgetvalue( res_s, i, j ) );
      else
        printf( "%-12s", PQgetvalue( res_s, i, j ) );
    }
    printf( "\n" );

    /* Значение поля record_book берем из текущей строки выборки, сделанной 
       из таблицы students. Это значение мы подставим в формируемую команду
       для получения выборки из таблицы progress. Мы должны выбрать из
       таблицы progress только те записи, у которых значение ключевого поля
       record_book (внешний ключ) такое же, как и в текущей строке выборки
       из таблицы students. В качестве шаблона в форматной строке функции
       sprintf() используем %s, а не %d, поскольку функция PQgetvalue()
       возвращает значение в виде символьной строки. Функция PQfnumber()
       дает номер поля по его имени. */
    sprintf( command_p, "SELECT * FROM progress WHERE record_book = %s",
             PQgetvalue( res_s, i, PQfnumber( res_s, "record_book" ) ) );

    res_p = PQexec( conn, command_p );

    if ( PQresultStatus( res_p ) != PGRES_TUPLES_OK )
    {
      fprintf( stderr, "Запрос SELECT к таблице progress неудачен: %s",
               PQerrorMessage( conn ) );
      /* Освободим память, занимаемую обеими структурами типа PGresult,
         одна из которых предназначена для работы с таблицей students,
         а другая -- progress. */
      PQclear( res_s );
      PQclear( res_p );
      exit_nicely( conn );
    }

    /* количество полей в выборке из таблицы progress */
    nFields_p = PQnfields( res_p );

    /* Выведем данные из полученной выборки
       (функция PQntuples возвращает число строк в выборке). */
    for ( k = 0; k < PQntuples( res_p ); k++ )
    {
      /* Значения полей из таблицы progress сместим немного вправо
         для наглядности (просто добавим пробелы в форматной строке). */
      printf( "    " );

      /* Перебираем все имена полей (столбцов) из таблицы progress. */
      for ( l = 0; l < nFields_p; l++ )
      {
        /* Для поля "Учебная дисциплина" сделаем более широкий столбец. */
        if ( strcmp( PQfname( res_p, l ), "subject" ) == 0 )
          /* Функция PQgetvalue возвращает значение поля номер l
             из строки номер k. */
          printf( "%-30s", PQgetvalue( res_p, k, l ) );
        else
          printf( "%-10s", PQgetvalue( res_p, k, l ) );
      }
      printf( "\n" );
    }

    printf( "\n" );
    PQclear( res_p );
  }

  /* Освободим память, которую занимала структура res_s,
     как только эта структура больше не нужна. */
  PQclear( res_s );

  /* Закроем соединение с базой данных. */
  PQfinish( conn );

  return 0;
}
