#!/usr/bin/perl -w
# ----------------------------------------------------------------------------
# Программа test_pq.pl.
# Организация доступа к базе данных с помощью пакетов DBI и DBD::Pg.
# ----------------------------------------------------------------------------

# Эта директива означает строгую проверку синтаксиса исходного
# текста программы.
use strict;

# Эта директива означает, что исходный текст скрипта представлен 
# в Unicode в кодировке UTF-8 (речь идет о символьных строках).
# Условие if означает, что подключение модуля utf8 будет выполнено,
# если операционная система -- Lnux.
use if ( $^O eq 'linux' ), "utf8";

# Этот модуль нужен для использования функции decode().
use if ( $^O eq 'linux' ), "Encode";

# Модуль для работы с базами данных.
use DBI;

# Так можно объявить константу.
use constant DBNAME => 'test';   # имя базы данных

# Назначим вывод в кодировке UTF-8 для устройства STDERR (стандартное 
# устройство вывода сообщений об ошибках), т. к. функция die() выводит
# сообщение на STDERR (см. perldoc -f die).
# Попробуйте закомментировать эту команду. Будет выведено предупреждение
# "Wide character in die at ...".
# ПРИМЕЧАНИЕ. Можно сделать и так (хотя эти два метода не тождественны):
#             binmode STDOUT, ':utf8';
binmode STDERR, ':encoding(UTF-8)';

# При выводе данных на стандартный вывод также назначаем кодировку UTF-8.
# Если эту строку закомментировать, то при вызове функций print будет
# выведено предупреждение "Wide character in print ...".
binmode STDOUT, ':encoding(UTF-8)';

# Попробуйте закомментировать эту строку и выполнить ввод записей
# в базу данных. Что получается в результате?
binmode STDIN, ':encoding(UTF-8)';

my $conn;     # дескриптор соединения с БД
my $rc;       # результат выполнения SQL-запроса
my $row;      # ссылка на хеш-массив для хранения одной записи из выборки
my $command;  # строка SQL-запроса для работв с БД
my $sth;      # результат подготовки запроса к БД
my $num_recs; # число записей, введенных в таблицу Students
my $err_flag; # признак наличия ошибки в рамках транзакции

# Сначала выполним подключение к базе данных.
# Параметры означают следующее: RaiseError => 0 -- будем сами проверять
# результат выполнения каждой операции с базой данных; AutoCommit => 1 --
# каждая операция автоматически фиксируется в базе данных, если только
# не была начата транзакция явным образом; PrintError => 0 -- не выводить
# дополнительную информацию об ошибке.
# Если значение RaiseError будет равно 1, тогда не нужно проверять
# результат выполнения каждой операции: в случае сбоя будет 
# сформировано исключение.
$conn = DBI->connect( "dbi:Pg:dbname=" . DBNAME, 'postgres', '',
                      { RaiseError => 0, AutoCommit => 1,
                        PrintError => 0 } );

# Проверим успешность подключения.
die "Не могу подключиться к базе данных " . DBNAME . ": " . 
    ( $^O eq 'linux' ) ? decode( 'UTF-8', $DBI::errstr ) : $DBI::errstr 
    unless defined $conn;

# Сначала добавим несколько записей в таблицу Students.
# В качестве иллюстрации работы с транзакциями выполним ввод записей
# в рамках единой транзакции. 
# ПРИМЕЧАНИЕ. Можно выполнение операции и проверку ее успешности
#             объединить с помощью логической операции or (пишется
#             строчными буквами!).
$rc = $conn->begin_work or
die "Не удалось начать транзакцию: " . $DBI::errstr;

# Сформируем SQL-запрос в виде символьной строки.
# Знаки "?" (placeholders) соответствуют фактическим значениям,
# которые будут переданы впоследствии методу execute.
$command = "INSERT INTO students " .
           "( record_book, name, psp_ser, psp_num ) " .
           "VALUES ( ?, ?, ?, ? )";

# Подготовим (prepare) эту операцию ввода записей в таблицу. Это позволит
# затем выполнять ее повторно, подставляя разные значения полей.
# При этом затраты ресурсов сервера уменьшаются.
$sth = $conn->prepare( $command ) or
die "Не удалось подготовить запрос: " . $DBI::errstr;

print "Вводите данные о студенте в одну строку,\n";
print "разделяя значения полей пробелами:\n";
print "Номер зачетной книжки Ф. И. О. Серия паспорта Номер паспорта\n";
print "Например:\n";
print "12345 Котов Петр Андреевич 0205 123098\n";
print "Для отказа от дальнейшего ввода просто нажмите клавишу Enter\n";

$num_recs = 0;
$err_flag = 0;

# Получаем данные со стандартного ввода.
while ( <STDIN> )
{
  chomp;
  last if /^$/;  # Если пустая строка -- завершаем ввод записей.

  # Разделим единую строку на отдельные поля.  
  my @values = split( /\s+/ );
  
  # Выполним запрос к базе данных.
  # Если произошла ошибка при выполнении запроса к базе данных,
  # то не будем завершать работу с помощью вызова функции die,
  # а в качестве примера обработаем эту ошибку сами.
  $rc = $sth->execute( $values[ 0 ],
                       "$values[ 1 ] $values[ 2 ] $values[ 3 ]",
                       $values[ 4 ], $values[ 5 ] );

  if ( defined $rc )  # Запись успешно добавлена.
  {
    $num_recs++;
    print "Запись добавлена в таблицу Students\n";
    print "Для отказа от ввода просто нажмите клавишу Enter\n";
  }
  else
  {
    # Можно получить сведения об ошибке не только с помощью $DBI->errstr,
    # но и с помощью обращения к $sth. Переменная state содержит код
    # ошибки. Такие коды предусмотрены стандартом SQL.
    print "Не удалось добавить запись в таблицу Students\n";
    print "Ошибка (код PostgreSQL): " . $sth->state . "\n";
    print $sth->errstr . "\n\n";
    $err_flag = 1;
    last;
  }
}

# Если ввели хотя бы одну запись и не произошло ошибки в процессе
# выполнения транзакции.
if ( $num_recs > 0 && ! $err_flag )
{
  print "Сохранить записи в базе данных (y/n)?\n";

  my $save = <STDIN>;
  chomp $save;

  if ( $save eq 'y' )
  {
    # Зафиксируем транзакцию в базе данных.
    $rc = $conn->commit or
    die "Не удалось зафиксировать транзакцию: " . $DBI::errstr;
  }
  else
  {
    print "Отменим транзакцию\n\n";
    $rc = $conn->rollback or 
    die "Не удалось отменить транзакцию: " . $DBI::errstr;
  }
}
# Не ввели ни одной записи или была ошибка в транзакции.
else
{
  print "Отменим транзакцию\n\n";
  $rc = $conn->rollback or 
  die "Не удалось отменить транзакцию: " . $DBI::errstr;
}

# Выведем все строки из таблицы Students, упорядочим их по значениям
# поля name. Сформируем SQL-запрос в виде символьной строки.
$command = "SELECT * FROM students ORDER BY name";

# Подготовим запрос.
$sth = $conn->prepare( $command ) or
die "Не удалось подготовить запрос: " . $DBI::errstr;

# Выполним запрос к базе данных.
$rc = $sth->execute or
die "Не удалось выполнить запрос: " . $DBI::errstr;

# Если в выборке нет ни одной строки, завершим работу.
if ( $sth->rows == 0 )
{
  print "В таблице Students нет записей\n";

  # Отключаемся от сервера баз данных.
  $conn->disconnect or
  die "Сбой при отключении от сервера: " . $DBI::errstr;

  exit ( 0 );
}

# Выведем заголовки столбцов.
printf "%-12s", "Зач. книжка";
printf "%-30s", "Ф. И. О.";
printf "%-12s", "Серия пасп.";
printf "%-12s", "Номер пасп.";
print "\n\n";

# Выводим все строки из полученной выборки.
# Метод fetchrow_hashref считывает в хеш-массив очередную строку
# из выборки и возвращает ссылку на этот хеш-масив.
while ( $row = $sth->fetchrow_hashref )
{
  # Значения NULL, которые могут присутствовать в базе данных,
  # представляются в Perl в виде значений undef. Для корректного вывода
  # данных на экран заменим значения undef пустыми строками.
  foreach ( keys %{ $row } )
  {
    $row->{ $_ } = '' unless defined $row->{ $_ };
  }

  # Обращаемся к хеш-массиву, используя в качестве ключей имена
  # полей (столбцов) таблицы Students.
  printf "%-12s", $row->{ 'record_book' };
  printf "%-30s", $row->{ 'name' };
  printf "%-12s", $row->{ 'psp_ser' };
  printf "%-12s", $row->{ 'psp_num' };
  print "\n";
}

# Отключаемся от сервера баз данных.
$conn->disconnect or
die "Сбой при отключении от сервера: " . $DBI::errstr;

exit ( 0 );
