<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  работа с базой данных (получение и сохранение информации 
           об учебных дисциплинах)
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

// Строку параметров для подключения к базе данных определим в виде
// константы.
define( "CONNECTION_STRING", "host=localhost dbname=edu user=postgres" );

// Поместив подобную конструкцию в каждую процедуру, можно выяснить,
// какие процедуры работают при выполнении того или иного запроса.
// $log = fopen( 'log.txt', 'a' );
// fwrite( $log, "insert_discipline\n" );
// fclose( $log );

/*
  Удаление одной записи из таблицы disciplines.
*/
function delete_discipline( $discipline_code )
{
  // Параметр: $discipline_code -- код учебной дисциплины.

  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "DELETE FROM disciplines WHERE discipline_code = $1";

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query, array( $discipline_code ) );

  // ПРИМЕЧАНИЕ. Если будет необходимо получать код SQLSTATE, который СУБД
  //             возвращает в случае ошибки, тогда нужно использовать
  //             такую функцию:
  // pg_send_query_params( $dbconn, $query, array( $discipline_code ) );
  //             Получить результат можно будет так:
  // $result = pg_get_result( $dbconn );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    // Просто отправим сообщение об ошибке на стандартный вывод. Оно будет
    // отослано web-сервером браузеру и обработано в функции delete_record
    // (см. файл university.js).
    echo pg_last_error( $dbconn );

    // ПРИМЕЧАНИЕ. Если бы мы захотели сами формировать сообщение об ошибке
    //             в зависимости от значения кода SQLSTATE, тогда нужно
    //             было бы воспользоваться этой функцией:
    // echo pg_result_error_field( $result, PGSQL_DIAG_SQLSTATE );
    //             Тогда мы должны были бы освободить память:
    // pg_free_result( $result );

    // Закроем соединение с сервером баз данных.
    pg_close( $dbconn );

    return;
  }

  // Сформируем объект JSON. Его поле rows будет содержать число фактически
  // удаленных записей (1 или 0). Выведем объект JSON на стандартный вывод.
  // Таким способом web-сервер передаст это значение функции delete_record,
  // написанной на языке JavaScript (см. файл university.js). Эта функция
  // вызывается из функции delete_discipline при выборе ссылки "Удалить"
  // (см. функцию display_disciplines_list в файле disciplines_view.php).
  echo '{ "rows": "' . pg_affected_rows( $result ) . '" }';
  
  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );
}


/*
  Выборка одной записи из таблицы disciplines.
*/
function get_discipline_info( $discipline_code )
{
  // Параметр: $discipline_code -- код учебной дисциплины.
  
  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Выберем данные для одной дисциплины.
  $query = "SELECT discipline_code, discipline_name " .
           "FROM disciplines WHERE discipline_code = $1";

  $result = pg_query_params( $dbconn, $query, array( $discipline_code ) );

  // Обратите внимание, что используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Получим данные из выборки -- это всего одна запись.
  $discipline_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC );
  
  pg_free_result( $result );
  pg_close( $dbconn );

  return $discipline_info;
}

/*
  Выборка всех записей из таблицы disciplines.
*/
function get_disciplines_list()
{
  $disciplines_list = array();

  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = 'SELECT discipline_code, discipline_name ' .
           'FROM disciplines ORDER BY discipline_name';

  // Поскольку условия WHERE в запросе нет, воспользуемся функцией pg_query.
  $result = pg_query( $dbconn, $query );

  // Обратите внимание, что используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Выбираем из полученной выборки строки одну за другой в цикле.
  // Параметр NULL означает, что берется следующая строка из выборки.
  // Параметр PGSQL_ASSOC означает, что строка из таблицы будет представлена
  // в виде хеш-массива (ассоциативного массива).
  while ( $discipline_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC ) )
  {
    // Отсутствие индекса (пустые квадратные скобки) означает, 
    // что новое значение будет добавлено в конец массива.
    // Получаем двухмерный массив, поскольку $discipline_info -- это также
    // массив (ассоциативный).
    $disciplines_list[] = $discipline_info;
  }

  pg_free_result( $result );
  pg_close( $dbconn );

  return $disciplines_list;
}

/*
  Добавление одной записи в таблицу disciplines.
*/
function insert_discipline()
{
  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "INSERT INTO disciplines ( discipline_code, discipline_name ) " .
           "VALUES ( $1, $2 )";

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'discipline_code' ],
                                    $_POST[ 'discipline_name' ]
                                  )
                           );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически добавленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );
  
  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );

  return $num_rows;
}

/*
  Обновление одной записи в таблице disciplines.
*/
function update_discipline()
{
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = "UPDATE disciplines " .
           "SET ( discipline_code, discipline_name ) = " .
           "( $1, $2 ) " .
           "WHERE discipline_code = $3";

  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'discipline_code' ],
                                    $_POST[ 'discipline_name' ],
                                    $_POST[ 'current_discipline_code' ] )
                           );

  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически обновленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );

  pg_free_result( $result );
  pg_close( $dbconn );

  return $num_rows;
}

?>
