<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  "Экзаменационные оценки" (работа с базой данных)
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

// Строку параметров для подключения к базе данных определим в виде
// константы.
define( "CONNECTION_STRING", "host=localhost dbname=edu user=postgres" );

// Поместив подобную конструкцию в каждую процедуру, можно выяснить,
// какие процедуры работают при выполнении того или иного запроса.
// $log = fopen( 'log.txt', 'a' );
// fwrite( $log, "insert_exam " . $record_book . "\n" );
// fclose( $log );

/*
  Удаление одной записи из таблицы progress.
*/
function delete_exam( $record_book, $acad_year, $term, $discipline_code )
{
  // Параметры: $record_book -- номер зачетной книжки;
  //            $aad_year -- учебный год;
  //            $term -- семестр;
  //            $dscipline_code -- код учебной дисциплины.

  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "DELETE FROM progress " .
           "WHERE record_book = $1 AND acad_year = $2 AND " .
                 "term = $3 AND discipline_code = $4" ;

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query,
                             array( $record_book, $acad_year, $term,
                                    $discipline_code )
                           );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    // Просто отправим сообщение об ошибке на стандартный вывод. Оно будет
    // отослано web-сервером браузеру и обработано в функции delete_record
    // (см. файл university.js).
    echo pg_last_error( $dbconn );

    // Закроем соединение с сервером баз данных.
    pg_close( $dbconn );

    return;
  }

  // Сформируем объект JSON. Его поле rows будет содержать число фактически
  // удаленных записей (1 или 0). Выведем объект JSON на стандартный вывод.
  // Таким способом web-сервер передаст это значение функции delete_record,
  // написанной на языке JavaScript (см. файл university.js). Эта функция
  // вызывается из функции delete_exam при выборе ссылки "Удалить"
  // (см. функцию display_exams_list в файле progress_view.php).
  echo '{ "rows": "' . pg_affected_rows( $result ) . '" }';

  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );
}

/*
  Выборка одной записи из таблицы progress.
*/
function get_exam_info( $record_book, $acad_year, $term, $discipline_code )
{
  // Параметры: $record_book -- номер зачетной книжки;
  //            $acad_year -- учебный год;
  //            $term -- семестр;
  //            $discipline_code -- код учебной дисциплины.
  
  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Выберем данные для одного студента.
  $query = "SELECT p.record_book, p.acad_year, p.term, " .
                  "p.discipline_code, d.discipline_name, p.mark " .
           "FROM progress p JOIN disciplines d " .
                "ON p.discipline_code = d.discipline_code " .
           "WHERE p.record_book = $1 AND p.acad_year = $2 AND " .
                 "p.term = $3 AND p.discipline_code = $4";

  $result = pg_query_params( $dbconn, $query,
                             array( $record_book, $acad_year, $term, 
                                    $discipline_code )
                           );

  // Обратите внимание, что используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Получим данные из выборки -- это всего одна запись.
  $exam_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC );
  
  pg_free_result( $result );
  pg_close( $dbconn );

  return $exam_info;
}

/*
  Выборка всех записей из таблицы progress для студента с указанным 
  номером зачетной книжки.
*/
function get_exams_list( $record_book )
{
  // Параметр: $record_book -- номер зачетной книжки.

  $exams_list = array();

  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = "SELECT p.record_book, p.acad_year, p.term, " .
                  "p.discipline_code, d.discipline_name, p.mark " .
           "FROM progress p JOIN disciplines d ON " .
                "p.discipline_code  = d.discipline_code " .
           "WHERE record_book = $1 " .
           "ORDER BY p.acad_year, p.term, d.discipline_name";

  $result = pg_query_params( $dbconn, $query, array( $record_book ) );

  // Обратите внимание, что теперь используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Выбираем из полученной выборки строки одну за другой в цикле.
  // Параметр NULL означает, что берется следующая строка из выборки.
  // Параметр PGSQL_ASSOC означает, что строка из таблицы будет представлена
  // в виде хеш-массива (ассоциативного массива).
  while ( $exam_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC ) )
  {
    // Отсутствие индекса (пустые квадратные скобки) означает, 
    // что новое значение будет добавлено в конец массива.
    // Получаем двухмерный массив, поскольку $exam_info -- это также
    // массив (ассоциативный).
    $exams_list[] = $exam_info;
  }

  pg_free_result( $result );
  pg_close( $dbconn );

  return $exams_list;
}

/*
  Добавление одной записи в таблицу progress.
*/
function insert_exam()
{
  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "INSERT INTO progress " .
                  "( record_book, acad_year, term, discipline_code, mark ) " .
           "VALUES ( $1, $2, $3, $4, $5 )";

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'record_book' ],
                                    $_POST[ 'acad_year' ],
                                    $_POST[ 'term' ],
                                    $_POST[ 'discipline_code' ],
                                    $_POST[ 'mark' ] )
                           );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически добавленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );
  
  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );

  return $num_rows;
}

/*
  Обновление одной записи в таблице progress.
*/
function update_exam()
{
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = "UPDATE progress " .
           "SET ( record_book, acad_year, term, discipline_code, mark ) = " .
           "( $1, $2, $3, $4, $5 ) " .
           "WHERE record_book = $6 AND acad_year = $7 AND " .
                 "term = $8 AND discipline_code = $9";

  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'record_book' ],
                                    $_POST[ 'acad_year' ],
                                    $_POST[ 'term' ],
                                    $_POST[ 'discipline_code' ],
                                    $_POST[ 'mark' ],
                                    $_POST[ 'current_record_book' ],
                                    $_POST[ 'current_acad_year' ],
                                    $_POST[ 'current_term' ],
                                    $_POST[ 'current_discipline_code' ]
                                  )
                           );

  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически обновленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );

  pg_free_result( $result );
  pg_close( $dbconn );

  return $num_rows;
}

?>
