<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  работа с базой данных (получение и сохранение информации 
           о студентах)
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

// Строку параметров для подключения к базе данных определим в виде
// константы.
define( "CONNECTION_STRING", "host=localhost dbname=edu user=postgres" );

// Поместив подобную конструкцию в каждую процедуру, можно выяснить,
// какие процедуры работают при выполнении того или иного запроса.
// $log = fopen( 'log.txt', 'a' );
// fwrite( $log, "delete_student " . $record_book . "\n" );
// fclose( $log );

/*
  Удаление одной записи из таблицы students.
*/
function delete_student( $record_book )
{
  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "DELETE FROM students WHERE record_book = $1";

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query, array( $record_book ) );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    // Просто отправим сообщение об ошибке на стандартный вывод. Оно будет
    // отослано web-сервером браузеру и обработано в функции delete_record
    // (см. файл university.js).
    echo pg_last_error( $dbconn );

    // Закроем соединение с сервером баз данных.
    pg_close( $dbconn );

    return;
  }

  // Сформируем объект JSON. Его поле rows будет содержать число фактически
  // удаленных записей (1 или 0). Выведем объект JSON на стандартный вывод.
  // Таким способом web-сервер передаст это значение функции delete_record,
  // написанной на языке JavaScript (см. файл university.js). Эта функция
  // вызывается из функции delete_student при выборе ссылки "Удалить"
  // (см. функцию display_students_list в файле students_view.php).
  echo '{ "rows": "' . pg_affected_rows( $result ) . '" }';

  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );
}

/*
  Выборка одной записи из таблицы students.
*/
function get_student_info( $record_book )
{
  // Параметр: $record_book -- номер зачетной книжки
  
  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Выберем данные для одного студента.
  $query = "SELECT record_book, name, psp_ser, psp_num " .
           "FROM students WHERE record_book = $1";

  $result = pg_query_params( $dbconn, $query, array( $record_book ) );

  // Обратите внимание, что теперь используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Получим данные из выборки -- это всего одна запись.
  $student_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC );
  
  pg_free_result($result);
  pg_close( $dbconn );

  return $student_info;
}

/*
  Выборка всех записей из таблицы students.
*/
function get_students_list()
{
  $students_list = array();

  $dbconn = pg_connect( CONNECTION_STRING ) or
    die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = 'SELECT record_book, name, psp_ser, psp_num ' .
           'FROM students ORDER BY name';

  // Поскольку условия WHERE в запросе нет, воспользуемся функцией pg_query.
  $result = pg_query( $dbconn, $query );

  // Обратите внимание, что теперь используется статус PGSQL_TUPLES_OK,
  // а не PGSQL_COMMAND_OK.
  if ( pg_result_status( $result ) != PGSQL_TUPLES_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Выбираем из полученной выборки строки одну за другой в цикле.
  // Параметр NULL означает, что берется следующая строка из выборки.
  // Параметр PGSQL_ASSOC означает, что строка из таблицы будет представлена
  // в виде хеш-массива (ассоциативного массива).
  while ( $student_info = pg_fetch_array( $result, NULL, PGSQL_ASSOC ) )
  {
    // Отсутствие индекса (пустые квадратные скобки) означает, 
    // что новое значение будет добавлено в конец массива.
    // Получаем двухмерный массив, поскольку $student_info -- это также
    // массив (ассоциативный).
    $students_list[] = $student_info;
  }

  pg_free_result( $result );
  pg_close( $dbconn );

  return $students_list;
}

/*
  Добавление одной записи в таблицу students.
*/
function insert_student()
{
  // Подключаемся к базе данных.
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  // Сформируем запрос, в котором вместо фактических значений стоят
  // переменные-заменители: $1, $2 и т. д.
  $query = "INSERT INTO students ( record_book, name, psp_ser, psp_num ) " .
           "VALUES ( $1, $2, $3, $4 )";

  // Выполняем обращение к базе данных. Фактические значения полей передаем
  // в виде массива.
  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'record_book' ], $_POST[ 'name' ],
                                    $_POST[ 'psp_ser' ], $_POST[ 'psp_num' ] )
                           );

  // Проверим состояние полученного результата.
  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически добавленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );
  
  // Освободим память.
  pg_free_result( $result );

  // Закроем соединение с сервером баз данных.
  pg_close( $dbconn );

  return $num_rows;
}

/*
  Обновление одной записи в таблице students.
*/
function update_student()
{
  $dbconn = pg_connect( CONNECTION_STRING )
    or die( 'Не могу подключиться к базе данных: ' . pg_last_error() );

  $query = "UPDATE students " .
           "SET ( record_book, name, psp_ser, psp_num ) = " .
           "( $1, $2, $3, $4 ) " .
           "WHERE record_book = $5";

  $result = pg_query_params( $dbconn, $query,
                             array( $_POST[ 'record_book' ], $_POST[ 'name' ],
                                    $_POST[ 'psp_ser' ], $_POST[ 'psp_num' ],
                                    $_POST[ 'current_record_book' ] )
                           );

  if ( pg_result_status( $result ) != PGSQL_COMMAND_OK )
  {
    die ( 'Ошибка: ' . pg_last_error( $dbconn ) );
  }

  // Число фактически обновленных записей (1 или 0).
  $num_rows = pg_affected_rows( $result );

  pg_free_result( $result );
  pg_close($dbconn);

  return $num_rows;
}

?>
